package ps3;

import java.util.Vector;

public class StringGraphA implements StringGraph {
	private Vector<String> nodes;
	private boolean [][] edges;
	// Abstraction function:
	//   Nodes = { nodes[i] | 0 <= i < nodes.size () }
	//   Edges = { { nodes[a], nodes[b] } | forall 0 <= a, b < nodes.size() where edges[a][b] = true }
	// Rep Invariant:
	//   nodes != null; edges != null
    //   no duplicates in nodes
	//   edges.length >= nodes.size ()
	//   forall 0 <= i < nodes.size: edges[i].length = edges.length
	//     The last two clauses state that edges is a square matrix, at least
	//     as big as nodes (but possibly bigger).
	
	static final private int SIZE_INCREMENT = 4;
	
	public StringGraphA() {
		nodes = new Vector<String>();
		
		// note: Java default initializes boolean to false
		edges = new boolean [SIZE_INCREMENT][SIZE_INCREMENT]; 
	}

	private int lookupNode(String s) {
		// EFFECTS: if s is a node in this, returns i such that nodes.getAt(i).equals(s)
		//    Otherwise, returns -1.
		for (int i = 0; i < nodes.size(); i++) {
			if (s.equals(nodes.elementAt(i))) {
				return i;
			}
		}
		return -1;
	}

	private void expandEdges() {
		// Increases the size of the edges matrix (but does not modify abstract state)
		boolean [][] oldedges = edges;
		edges = new boolean[nodes.size() + SIZE_INCREMENT][nodes.size() + SIZE_INCREMENT];
		for (int i = 0; i < oldedges.length; i++) {
			for (int j = 0; j < oldedges.length; j++) {
				edges[i][j] = oldedges[i][j];
			}
		}
	}
	
	public void addEdge(String s, String t) throws NoNodeException,
			DuplicateException {
		int sindex = lookupNode(s);
		int tindex = lookupNode(t);
		if (sindex == -1) throw new NoNodeException(s);
		if (tindex == -1) throw new NoNodeException(t);
		if (edges[sindex][tindex]) throw new DuplicateException("Duplicate edge " + s + "->" + t);
		edges[sindex][tindex] = true;
	}

	public void addNode(String s) throws DuplicateException {
		if (lookupNode(s) != -1) {
			throw new DuplicateException(s);
		}
		nodes.add(s);
		if (edges.length < nodes.size()) {
			expandEdges();
		}
	}

	public Set<String> getAdjacent(String s) throws NoNodeException {
		Set<String> res = new Set<String>();
		int index = lookupNode(s);
		if (index == -1) throw new NoNodeException(s);
		for (int i = 0; i < nodes.size(); i++) {
			if (edges[index][i]) {
				res.add(nodes.elementAt(i));
			}
		}
		return res;
	}

	public String toString() {
		String res = "<";
		res += nodes.toString();
		res += "; ";
		boolean firstone = true;
		for (int i = 0; i < nodes.size(); i++) {
			for (int j = 0; j < nodes.size(); j++) {
				if (edges[i][j]) {
					if (firstone) {
						firstone = false;
					} else {
						res += ", ";
					}
					res += "<" + nodes.elementAt(i) + ", " + nodes.elementAt(j) + ">";
				}
			}
		}
		res += ">";
		return res;
	}
}
